
# tuner.py
#
# Python Digital Audio Recorder
#
# Copyright (C) 2003 Daniel Thompson <d.thompson@gmx.net> 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#

import os, string, sys, videodev

class Tuner:
	""" class to interact with the specific tuner backend """

	def __init__(self, prefs, scriptFile):
		self.prefs = prefs
		self.scriptFile = scriptFile

	def get_id_list(self):
		""" generate a list of valid ids to allow the list to be pruned """
		cout = os.popen("atq", 'r')
		jobs = cout.readlines()
		cout.close()

		list = []

		for job in jobs:
			id = string.split(job)[0]
			list.append(string.atoi(id))

		return list

	def add_programme(self, programme):
		""" generate a job to cause the programme to be recorded """

		# unpack the tuple argument
		station, (dd, mm, yy), (HH, MM), (eHH, eMM), fname = programme

		# calculate the number of seconds that has been requested.
		if eHH < HH or (eHH == HH and eMM <= MM): eHH += 24
		seconds = (((eHH * 60) + eMM) * 60) - (((HH * 60) + MM) * 60)

		cmd = "at %02d:%02d %02d.%02d.%02d" % (HH, MM, dd, mm, yy)
		cin, cout = os.popen4(cmd)

		# record at CD quality with two channels
		rec = 'sox -tossdsp -r44100 -sw -c2 /dev/dsp'

		# add any compression that the file extender requests
		if fname.endswith('.ogg'):
			quality = self.prefs.get_preference('vorbis_quality')
			rec += ' -twav - 2> /dev/null | ' \
			       'oggenc --quiet --quality=%f - --output=' % quality
		elif fname.endswith('.spx'):
			quality = self.prefs.get_preference('speex_quality')
			rate = self.prefs.get_preference('speex_rate')
			# to compress with speex we will downsample to one of its optimized
			# sample rates before encoding
			rec += ' -twav -r%s - resample 2> /dev/null | ' \
			       'speexenc --vbr --quality=%d - ' % (rate, quality)
		else:
			# any unknown extension will be stored in Microsoft WAV format
			rec += ' -twav '

		rec += fname
		#print rec

		# generate a shell script that:
		#  - tunes to the requested radio station
		#  - ?sets up the mixer?
		#  - starts recording
		#  - sleeps for some period of time
		#  - stops recording and exits
		cin.write("""
# configure the hardware to do what needs to be done
pydar=%s
$pydar --tune="%s"
#echo TODO: set the mixer to the radio source

%s > /dev/null 2>&1 &
pid=$!

# wait for the recording to finish
sleep %d

# stop the recording
$pydar --mute
kill $!
"""			% (self.scriptFile, station, rec, seconds))
		cin.close()

		# suck up all the input until we have a job id
		out = ''
		while -1 == out.find('job'):
			out = cout.readline()
		out = out.split()
			
		cout.close()

		# return the second word (the job id) as an integer
		return string.atoi(out[1])

	def delete_programme(self, id):
		""" remove the specified job """
		os.system("atrm %d" % id)

	def is_videodev_usable(self):
		try:
			vd = self.open_videodev()
			vd.close()
		except IOError:
			return False
		return True

	def open_videodev(self):
		radio = self.prefs.get_preference('radio')
		return videodev.VideoDev(radio)

	def tune(self, channel):
		vd = self.open_videodev()

		# tune to the requested station
		try:
			freq = self.prefs.get_station_by_name(channel)
		except IndexError:
			freq = string.atof(channel)

		freq = (freq + (1.0/32.0)) * 16
		if 'VIDEO_TUNER_LOW' in vd.get_tuner()['flags']:
			freq = freq * 1000
		vd.set_freq(freq)
		
		# unmute the device
		aud = vd.get_audio()
		aud['volume'] = 65535
		if 'VIDEO_AUDIO_MUTE' in aud['flags']:
			aud['flags'].remove('VIDEO_AUDIO_MUTE')
		vd.set_audio(aud)

		vd.close()
	
	def mute(self):
		vd = self.open_videodev()

		# tune to nothing
		vd.set_freq(0)

		# set volume to 0 and mute
		aud = vd.get_audio();
		aud['volume'] = 0
		aud['flags'].append('VIDEO_AUDIO_MUTE')
		vd.set_audio(aud)

		vd.close()
